searchData={"content_type":"text/markdown","items":[{"doc":"Design By Contract for Elixir.\n\nAs described on [Wikipedia](https://en.wikipedia.org/wiki/Design_by_contract):\n\n> Design by contract (DbC), also known as contract programming, programming by contract and\n> design-by-contract programming, is an approach for designing software.\n>\n> It prescribes that software designers should define formal, precise and verifiable interface\n> specifications for software components, which extend the ordinary definition of\n> [abstract data types](https://en.wikipedia.org/wiki/Abstract_data_type) with preconditions,\n> postconditions and invariants. These specifications are referred to as \"contracts\", in\n> accordance with a conceptual metaphor with the conditions and obligations of business\n> contracts.\n>\n> The term was coined by [Bertrand Meyer](https://en.wikipedia.org/wiki/Bertrand_Meyer) in\n> connection with his design of the\n> [Eiffel programming language](https://en.wikipedia.org/wiki/Eiffel_(programming_language))\n> and first described in various articles starting in 1986 and the two successive editions\n> (1988, 1997) of his book\n> [_Object-Oriented Software Construction_](https://en.wikipedia.org/wiki/Object-Oriented_Software_Construction).\n>\n> Design by contract has its roots in work on\n> [formal verification](https://en.wikipedia.org/wiki/Formal_verification),\n> [formal specification](https://en.wikipedia.org/wiki/Formal_specification) and\n> [Hoare logic](https://en.wikipedia.org/wiki/Hoare_logic).\n\n`Bond` applies the central ideas of contract programming to Elixir and provides support for\nattaching preconditions and postconditions to function definitions and conditionally evaluating\nthem based on compile-time configuration.","ref":"Bond.html","title":"Bond","type":"module"},{"doc":"`use Bond` in your module and then define preconditions and postconditions for your functions\nwith the `@pre` and `@post` annotations, respectively. For example:\n\n```elixir\ndefmodule Math do\n  use Bond\n\n  @pre numeric_x: is_number(x), non_negative_x: x >= 0\n  @post float_result: is_float(result),\n        non_negative_result: result >= 0.0,\n        \"sqrt of 0 is 0\": (x == 0) ~> (result === 0.0),\n        \"sqrt of 1 is 1\": (x == 1) ~> (result === 1.0),\n        \"x > 1 implies result smaller than x\": (x > 1) ~> (result < x)\n  def sqrt(x), do: :math.sqrt(x)\nend\n```","ref":"Bond.html#module-quick-start","title":"Quick start - Bond","type":"module"},{"doc":"> #### `use Bond` {: .info}\n>\n> When you `use Bond`, the `Bond` module will override several `Kernel` macros in order to\n> support attaching preconditions and postconditions to functions. Specifically:\n>\n>   * `Kernel.@/1` is overridden by `Bond.@/1`\n>   * `Kernel.def/2` is overridden by `Bond.def/2`\n>   * `Kernel.defp/2` is overridden by `Bond.defp/2`\n>\n> `use Bond` will also import the `Bond` module so that the `check/1` and `check/2` macros are\n> available for use.\n>\n> Additionally, the `Bond.Predicates` module is automatically imported for all preconditions,\n> postconditions, and checks, so that the predicate functions and operators that are defined\n> therein can be used for assertions. `Bond.Predicates` can be explicitly imported into modules\n> for use outside of assertions.","ref":"Bond.html#module-usage","title":"Usage - Bond","type":"module"},{"doc":"Assertions in Bond are conditional Elixir expressions, optionally associated with a textual\nlabel (either an atom or a string). These assertions may appear in `@pre` or `@post`\nexpressions, or in calls to `check/1` or `check/2`.\n\nBond offers considerable flexibility in its assertion syntax; assertions may take any of the\nfollowing forms:\n\n  * `expression` - a \"bare\" expression without any associated label\n  * `label, expression` - an expression preceded by a string or atom label\n  * `expression, label` - an expression followed by a string or atom label\n  * `label_1: expression_1, label_2: expression_2` - a keyword list with labels as the keys and\n    expressions as the associated values","ref":"Bond.html#module-assertion-syntax","title":"Assertion syntax - Bond","type":"module"},{"doc":"Override `Kernel.@/1` to support `@pre` and `@post` annotations.\n\nSee the `Bond` module docs for the syntax of `@pre` and `@post` annotations.","ref":"Bond.html#@/1","title":"Bond.@/1","type":"macro"},{"doc":"Check an assertion or a keyword list of assertions for validity.\n\nReturns the result(s) of the assertion(s) if satisfied, or raises a `Bond.CheckError` exception\nif any assertions are not satisfied.","ref":"Bond.html#check/1","title":"Bond.check/1","type":"macro"},{"doc":"iex> check 1 == 1.0\n    true\n    iex> check 1 == 1.0, \"integer 1 is equal to float 1.0\"\n    true\n    iex> check \"integer 1 is equal to float 1.0\", 1 == 1.0\n    true\n    iex> check tautology: 1 == 1\n    [true]\n    iex> check \"1 is 1\": 1 == 1, \"2 is 2\": 2 == 2\n    [true, true]","ref":"Bond.html#check/1-examples","title":"Examples - Bond.check/1","type":"macro"},{"doc":"Check a single labelled assertion for validity.\n\nSee `check/1` for details and examples.","ref":"Bond.html#check/2","title":"Bond.check/2","type":"macro"},{"doc":"Override `Kernel.def/2` to support wrapping with preconditions and postconditions.","ref":"Bond.html#def/2","title":"Bond.def/2","type":"macro"},{"doc":"Override `Kernel.defp/2` to support wrapping with preconditions and postconditions.","ref":"Bond.html#defp/2","title":"Bond.defp/2","type":"macro"},{"doc":"Type to represent a compile-time quoted assertion expression, which must be a valid Elixir\nexpression that, when unquoted, evaluates to a `t:boolean/0` or `t:as_boolean/1` value.","ref":"Bond.html#t:assertion_expression/0","title":"Bond.assertion_expression/0","type":"type"},{"doc":"Type to represent a label for an assertion, which must be a compile-time atom or string.","ref":"Bond.html#t:assertion_label/0","title":"Bond.assertion_label/0","type":"type"},{"doc":"Subset of `Macro.Env` struct that excludes fields that, according to the documentation, \"are\nprivate to Elixir's macro expansion mechanism\".","ref":"Bond.html#t:env/0","title":"Bond.env/0","type":"type"},{"doc":"Predicate functions and operators that are useful in contract specifications.\n\nTo use the operator versions of the predicates, this module must be imported in the using module.","ref":"Bond.Predicates.html","title":"Bond.Predicates","type":"module"},{"doc":"Logical implication: does `p` imply `q`?","ref":"Bond.Predicates.html#implies?/2","title":"Bond.Predicates.implies?/2","type":"function"},{"doc":"iex> implies?(true, true)\n    true\n    iex> implies?(true, false)\n    false\n    iex> implies?(false, true)\n    true\n    iex> implies?(false, false)\n    true","ref":"Bond.Predicates.html#implies?/2-examples","title":"Examples - Bond.Predicates.implies?/2","type":"function"},{"doc":"Logical exclusive or: is either `p` or `q` true, but not both?","ref":"Bond.Predicates.html#xor/2","title":"Bond.Predicates.xor/2","type":"function"},{"doc":"iex> xor(true, true)\n    false\n    iex> xor(true, false)\n    true\n    iex> xor(false, true)\n    true\n    iex> xor(false, false)\n    false","ref":"Bond.Predicates.html#xor/2-examples","title":"Examples - Bond.Predicates.xor/2","type":"function"},{"doc":"Logical exclusive or operator: `p ||| q` means `xor(p, q)`.\n\nNote that the `|||` operator has higher precedence than many other operators and it may be\nnecessary to parenthesize the expressions on either side of the operator to get the\nexpected result.","ref":"Bond.Predicates.html#%7C%7C%7C/2","title":"Bond.Predicates.|||/2","type":"function"},{"doc":"iex> true ||| true\n    false\n    iex> true ||| false\n    true\n    iex> false ||| true\n    true\n    iex> false ||| false\n    false\n    iex> x = 2\n    2\n    iex> y = 4\n    4\n    iex> (x - y < 0) ||| (y <= x)\n    true","ref":"Bond.Predicates.html#%7C%7C%7C/2-examples","title":"Examples - Bond.Predicates.|||/2","type":"function"},{"doc":"Logical implication operator: `p ~> q` means `implies?(p, q)`.\n\nNote that the `~>` operator has higher precedence than many other operators and it may be\nnecessary to parenthesize the expressions on either side of the operator to get the\nexpected result.","ref":"Bond.Predicates.html#~%3E/2","title":"Bond.Predicates.~>/2","type":"function"},{"doc":"iex> true ~> true\n    true\n    iex> true ~> false\n    false\n    iex> false ~> true\n    true\n    iex> false ~> false\n    true\n    iex> x = 2\n    2\n    iex> y = 4\n    4\n    iex> (x - y < 0) ~> (y > x)\n    true","ref":"Bond.Predicates.html#~%3E/2-examples","title":"Examples - Bond.Predicates.~>/2","type":"function"},{"doc":"Exception raised when a `Bond.check/2` assertion fails.","ref":"Bond.CheckError.html","title":"Bond.CheckError","type":"exception"},{"doc":"The `Bond.CheckError` exception type.","ref":"Bond.CheckError.html#t:t/0","title":"Bond.CheckError.t/0","type":"type"},{"doc":"Exception raised when a function postcondition fails.","ref":"Bond.PostconditionError.html","title":"Bond.PostconditionError","type":"exception"},{"doc":"The `Bond.PostconditionError` exception type.","ref":"Bond.PostconditionError.html#t:t/0","title":"Bond.PostconditionError.t/0","type":"type"},{"doc":"Exception raised when a function precondition fails.","ref":"Bond.PreconditionError.html","title":"Bond.PreconditionError","type":"exception"},{"doc":"The `Bond.PreconditionError` exception type.","ref":"Bond.PreconditionError.html#t:t/0","title":"Bond.PreconditionError.t/0","type":"type"},{"doc":"# Bond\n\nDesign By Contract for Elixir.","ref":"readme.html","title":"Introduction","type":"extras"},{"doc":"Bond provides support for contract programming (also known as\n\"Design By Contract\") for Elixir.\n\nAs described on [Wikipedia](https://en.wikipedia.org/wiki/Design_by_contract):\n\n> Design by contract (DbC), also known as contract programming, programming by\n> contract and design-by-contract programming, is an approach for designing\n> software.\n>\n> It prescribes that software designers should define formal, precise and\n> verifiable interface specifications for software components, which extend the\n> ordinary definition of\n> [abstract data types](https://en.wikipedia.org/wiki/Abstract_data_type) with\n> preconditions, postconditions and invariants. These specifications are referred\n> to as \"contracts\", in accordance with a conceptual metaphor with the conditions\n> and obligations of business contracts.\n>\n> The term was coined by\n> [Bertrand Meyer](https://en.wikipedia.org/wiki/Bertrand_Meyer) in connection\n> with his design of the\n> [Eiffel programming language](https://en.wikipedia.org/wiki/Eiffel_(programming_language))\n> and first described in various articles starting in 1986 and the two successive\n> editions (1988, 1997) of his book\n> [_Object-Oriented Software Construction_](https://en.wikipedia.org/wiki/Object-Oriented_Software_Construction).\n>\n> Design by contract has its roots in work on\n> [formal verification](https://en.wikipedia.org/wiki/Formal_verification),\n> [formal specification](https://en.wikipedia.org/wiki/Formal_specification) and\n> [Hoare logic](https://en.wikipedia.org/wiki/Hoare_logic).","ref":"readme.html#about-this-library","title":"About this library - Introduction","type":"extras"},{"doc":"`bond` can be installed by adding it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bond, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Introduction","type":"extras"},{"doc":"Documentation is generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm/bond) and be found at\n .","ref":"readme.html#documentation","title":"Documentation - Introduction","type":"extras"},{"doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html","title":"License","type":"extras"}]}